--=================GENERATOR CHECKER in VHDL============
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

--///////////////////// ENTITY of Guard /////////////////////
entity GUARD is
port(
CLK        : in  std_logic;
RESET      : in  std_logic;
RST    : in std_logic;
START    : in std_logic;
COUNT    : in std_logic_vector(2 downto 0);
ERROR     : out std_logic );

end entity GUARD;

--///////////////////ARCHITECTURE of GUARD /////////////////////
architecture FL_GUARD of GUARD is

signal   P9  : std_logic;
signal   P0  : std_logic;
signal   P1  : std_logic;
signal   P2  : std_logic;
signal   P3  : std_logic;
signal   P4  : std_logic;
signal   P5  : std_logic;
signal   P6  : std_logic;
signal   P7  : std_logic;

type t_state is (S0,S1,S2,S3,S4,S5,S6,S7,Serr);
signal present_state, next_state : t_state;

begin

P9 <= '1' when  RST = '1' and START = '0' and COUNT = "000"
    else '0';
P0 <= '1' when  COUNT = "000" and RST = '0' and START = '1'
    else '0';
P1 <= '1' when  COUNT = "001" and RST = '0' and START = '1'
    else '0';
P2 <= '1' when  COUNT = "010" and RST = '0' and START = '1'
    else '0';
P3 <= '1' when  COUNT = "011" and RST = '0' and START = '1'
    else '0';
P4 <= '1' when  COUNT = "100" and RST = '0' and START = '1'
    else '0';
P5 <= '1' when  COUNT = "101" and RST = '0' and START = '1'
    else '0';
P6 <= '1' when  COUNT = "110" and RST = '0' and START = '1'
    else '0';
P7 <= '1' when  COUNT = "111" and RST = '0' and START = '1'
    else '0';

-- FSM present state register -------------------------
sync_logic : process(RESET, CLK)
begin
  if (RESET = '1') then
     present_state <= S0;
  elsif (CLK'event AND CLK = '1') then
     present_state <= next_state;
  end if;
end process sync_logic;

-- FSM next state logic -------------------------------
next_state_logic : process(present_state, P9, P0, P1, P2, P3, P4, P5, P6, P7)
begin
  case present_state is
    when S0 =>
       if (P0 = '1') then
          next_state <= S1;
       else
          next_state <= Serr;
       end if;
    when S1 =>
       if (P1 = '1') then
          next_state <= S2;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when S2 =>
       if (P2 = '1') then
          next_state <= S3;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when S3 =>
       if (P3 = '1') then
          next_state <= S4;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when S4 =>
       if (P4 = '1') then
          next_state <= S5;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when S5 =>
       if (P5 = '1') then
          next_state <= S6;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when S6 =>
       if (P6 = '1') then
          next_state <= S7;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when S7 =>
       if (P7 = '1') then
          next_state <= S0;
       elsif (P9 = '1') then
          next_state <= S0;
       else
          next_state <= Serr;
       end if;
    when Serr =>
       next_state <= S0;
    when others =>
       next_state <= S0;
  end case;
end process next_state_logic;

-- FSM output logic------------------------------------
output_logic : process(present_state)
begin
  ERROR <= '0';
  case present_state is
     when Serr => ERROR <= '1';
     when others => ERROR <= '0';
  end case;
end process output_logic;

end architecture FL_GUARD;